<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Configuration
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2008-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


/**
 * Configuration class.
 *
 * Provides feature to load (include) multiple versions of a configuration file:
 * - loads original configuration file /path_to_config/myconfig.php
 * - loads userdefined version of configuration file /path_to_config/myconfig.user.php (overwrites existing myconfig.php)
 * - loads client configuration file /path_to_client/myconfig.php (overwrites existing myconfig.user.php)
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Configuration
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2008-2010 Murat Purc (http://www.purc.de)
 */
class ConMp_Config
{
    /**
     * Self instance
     * @var  ConMp_Config
     */
    private static $_instance = null;

    /**
     * Contenido configuration array
     * @var  array
     */
    protected $_cfg;


    protected function __construct()
    {
        if (!isset($GLOBALS['cfg'])) {
            $GLOBALS['cfg'] = array();
        }
        $this->_cfg = & $GLOBALS['cfg'];
    }


    private function __clone()
    {
        // donut
    }


    public static function getInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }


    public function __set($key, $value)
    {
        $this->set($key, $value);
    }


    public function __get($key)
    {
        if ($this->exists($key)) {
            return $this->_cfg[$key];
        }

        $trace = debug_backtrace();
        trigger_error(
            'Undefined property via __get(): ' . $key . ' in ' . $trace[0]['file'] . ' on line ' . $trace[0]['line'], E_USER_NOTICE
        );
        return null;
    }


    public function __isset($key)
    {
        return $this->exists($key);
    }


    public function __unset($key)
    {
        $this->remove($key);
    }


    public function set($key, $value)
    {
        $this->_cfg[$key] = $value;
    }


    public function get($key, $default=null)
    {
        return ($this->exists($key)) ? $this->_cfg[$key] : $default;
    }


    public function exists($key)
    {
        return isset($this->_cfg[$key]);
    }


    public function remove($key)
    {
        if ($this->exists($key)) {
            unset($this->_cfg[$key]);
        }
    }


    public function loadConfig($path, $config)
    {
        // load configuration file /path_to_config/myconfig.php
        $coreConfigFile = $path . $config . '.php';
        $coreOk = $this->_load($coreConfigFile);

        // load configuration file /path_to_config/myconfig.user.php (overwrites existing myconfig.php)
        $userConfigFile = $path . $config . '.user.php';
        $userOk = $this->_load($userConfigFile);

        // load client configuration file /path_to_client/myconfig.php (overwrites existing myconfig.user.php)
        $clientConfigFile = '';
        $clientOk = false;
        if (isset($GLOBALS['cfgClient']) && isset($GLOBALS['client'])) {
            $frontend = $GLOBALS['cfgClient'][$GLOBALS['client']]['path']['frontend'];
            $clientConfigFile = $frontend . $config . '.php';
            if ($clientConfigFile !== $coreConfigFile) {
                $clientOk = $this->_load($clientConfigFile);
            }
        }

        if (!$coreOk && !$userOk && !$clientOk) {
            throw new Exception("Couldn't load configuration file $coreConfigFile, $userConfigFile or $clientConfigFile");
        }
    }


    protected function _load($configFile)
    {
        global $cfg;
        if (is_file($configFile)) {
            include_once($configFile);
            return true;
        } else {
            return false;
        }
    }

/*
    protected function _toObject(array $data)
    {
        $object = new stdClass();
        if (count($data) > 0) {
            foreach ($data as $name => $value) {
                $name = self::getCleanName($name);
                if ($name !== '') {
                    if (is_array($value)) {
                        $object->$name = self::toObject($value);
                    } else {
                        $object->$name = $value;
                    }
                }
            }
        }
        return $object;
    }
*/
}
