<?php
/**
 * Contenido extension pack (ContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Frontend_Page
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2008-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


/**
 * Frontend page class
 *
 * @TODO: Need a Interface, a abstract class and create pages for HTML, XML and JSON container
 * @package     Plugin_mpContenidoX
 * @subpackage  Frontend_Page
 * @author      Murat Purc <murat@purc.de>
 */
class ConMp_FePage
{
    const HEADER = 'head';

    const FOOTER = 'footer';

    const MEDIA_ALL = 'all';

    const MEDIA_AURAL = 'aural';

    const MEDIA_BRAILLE = 'braille';

    const MEDIA_EMBOSSED = 'embossed';

    const MEDIA_HANDHELD = 'handheld';

    const MEDIA_PRINT = 'print';

    const MEDIA_PROJECTION = 'projection';

    const MEDIA_SCREEN = 'screen';

    const MEDIA_TTY = 'tty';

    const MEDIA_TV = 'tv';
    
    const META_NAME = 'name';

    const META_HTTP_EQUIV = 'http-equiv';

    private static $_instance;

    protected $_xhtml = true;

    protected $_baseHref = '';

    protected $_title = '';

    protected $_metas = array();

    protected $_scripts = array();

    protected $_jsCode = array();

    protected $_css = array();

    protected $_cssCode = array();

    protected $_moduleOutputs = array();


    /**
     * Prevent public instantiation
     */
    protected function __construct()
    {
        $this->_metas[self::META_NAME] = array();
        $this->_metas[self::META_HTTP_EQUIV] = array();

        $this->_scripts[self::HEADER] = array();
        $this->_scripts[self::FOOTER] = array();
        $this->_jsCode[self::HEADER] = array();
        $this->_jsCode[self::FOOTER] = array();
    }


    /**
     * Disable cloning
     */
    private function __clone()
    {
    }


    /**
     * Returns self instance
     *
     * @return  ConMp_FePage
     */
    public static function getInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }


    public function setXHTML($xhtml)
    {
        $this->_xhml = (bool) $xhtml;
    }

    
    public function getXHTML()
    {
        return $this->_xhml;
    }

    
    public function setTitle($title)
    {
        $this->_title = $title;
    }

    
    public function getTitle()
    {
        return $this->_title;
    }
    
    
    public function renderTitle($wrap="%s\n")
    {
        $tag = '<title>' . $this->_title . '</title>';
        return sprintf($wrap, $tag);
    }

    
    public function setBaseHref($baseHref)
    {
        $this->_baseHref = $baseHref;
    }

    
    public function getBaseHref()
    {
        return $this->_baseHref;
    }

    
    public function renderBaseHref($wrap="%s\n")
    {
        if ($this->_baseHref == '') {
            return '';
        }
        $tag = '<base href="' . $this->_baseHref . '"' . ($this->_xhtml) ? ' />' : '>';
        return sprintf($wrap, $tag);
    }

	
    public function setMeta($name, $content, $type='name')
	{
		if ($type !== self::META_HTTP_EQUIV) {
            $type = self::META_NAME;
        }

        $this->_metas[$type][$name] = (string) $content;
	}

	
    public function getMeta($name, $type='name')
	{
		if ($type !== self::META_HTTP_EQUIV) {
            $type = self::META_NAME;
        }

        if (isset($this->_metas[$type][$name])) {
            return $this->_metas[$type][$name];
        } else {
            return null;
        }
	}

	
    public function getMetas()
	{
        return $this->_metas[$type][$name];
	}

	
    public function getMetasByType($type='name')
	{
		if ($type !== self::META_HTTP_EQUIV) {
            $type = self::META_NAME;
        }

        return $this->_metas[$type];
	}

    
    public function renderMetas($wrap="%s\n")
    {
        $ending   = ($this->_xhtml) ? ' />' : '>';
        $metaTags = '';
        foreach ($this->_metas as $type => $metas) {
            foreach ($metas as $name => $value) {
                $item = '<meta ' . $type . '="' . $name . '" content="' . $value . '"' . $ending;
                $metaTags .= sprintf($wrap, $item);
            }
        }
        return $metaTags;
    }
    
    
    public function addScript($file, $addTo=null, $wrap="%s\n")
    {
        $addTo = ($addTo == self::FOOTER) ? self::FOOTER : self::HEADER;
        $this->_scripts[$addTo][] = array(
            'file' => $file,
            'wrap' => $wrap
        );
    }

    
    public function getScripts($for=null)
    {
        if ($for == self::HEADER) {
            return $this->_scripts[self::HEADER];
        } elseif ($for == self::FOOTER) {
            return $this->_scripts[self::FOOTER];
        } else {
            return $this->_scripts;
        }
    }


    public function renderScripts($for=null)
    {
        $for = ($for == self::FOOTER) ? self::FOOTER : self::HEADER;
        $jsTags = '';
        foreach ($this->_scripts[$for] as $p => $item) {
            $tag = '<script type="text/javascript" src="' . $item['file'] . '"></script>';
            $jsTags .= sprintf($item['wrap'], $tag);
        }
        return $jsTags;
    }


    public function addJsCode($code, $addTo=null, $wrap="%s\n")
    {
        $addTo = ($addTo == self::FOOTER) ? self::FOOTER : self::HEADER;
        $this->_jsCode[$addTo][] = array(
            'code' => $code,
            'wrap' => $wrap
        );
    }


    public function getJsCodes($for=null)
    {
        if ($for == self::HEADER) {
            return $this->_jsCode[self::HEADER];
        } elseif ($for == self::FOOTER) {
            return $this->_jsCode[self::FOOTER];
        } else {
            return $this->_jsCode;
        }
    }


    public function renderJsCode($for=null)
    {
        $for    = ($for == self::FOOTER) ? self::FOOTER : self::HEADER;
        $start  = '<script type="text/javascript">' . (($this->_xhtml) ? '<!-- // <![CDATA[' : '<!--') . "\n";
        $end    = (($this->_xhtml) ? '//]]> -->' : '// -->') . '</script>';
        $jsCode = '';
        foreach ($this->_jsCode[$for] as $p => $item) {
            $item['code'] = trim($item['code']);
            $code    = $start . $item['code'] . $end;
            $jsCode .= sprintf($item['wrap'], $code);
        }
        return $jsCode;
    }


    public function addCss($file, $media=null, $wrap="%s\n")
    {
        switch ($media) {
            case self::MEDIA_AURAL:
            case self::MEDIA_BRAILLE:
            case self::MEDIA_EMBOSSED:
            case self::MEDIA_HANDHELD:
            case self::MEDIA_PRINT:
            case self::MEDIA_PROJECTION:
            case self::MEDIA_SCREEN:
            case self::MEDIA_TTY:
            case self::MEDIA_TV:
                break;
            default:
                $media = self::MEDIA_ALL;
                break;
        }

        $this->_css[] = array(
            'file'  => $file,
            'media' => $media,
            'wrap'  => $wrap
        );
    }


    public function getCss()
    {
        return $this->_css;
    }


    public function renderCss()
    {
        $ending   = ($this->_xhtml) ? ' />' : '>';
        $linkTags = '';
        foreach ($this->_css as $p => $item) {
            $tag = '<link rel="stylesheet" type="text/css" href="' . $item['file'] . '" media="' . $item['media'] . '"' . $ending;
            $linkTags .= sprintf($item['wrap'], $tag);
        }
        return $linkTags;
    }


    public function addCssCode($code, $wrap="%s\n")
    {
        $this->_cssCode[] = array(
            'code' => $code,
            'wrap' => $wrap
        );
    }


    public function getCssCodes($for=null)
    {
        return $this->_cssCode;
    }


    public function renderCssCodes()
    {
        $start  = '<style type="text/css">' . (($this->_xhtml) ? '<!-- // <![CDATA[' : '<!--') . "\n";
        $end    = (($this->_xhtml) ? '//]]> -->' : '// -->') . '</style>';
        $cssCode = '';
        foreach ($this->_cssCode as $p => $item) {
            $item['code'] = trim($item['code']);
            $code     = $start . $item['code'] . $end;
            $cssCode .= sprintf($item['wrap'], $code);
        }
        return $cssCode;
    }


    public function addModuleOutput($name, $value)
    {
        $this->_moduleOutputs[$name] = $value;
    }


    public function getModuleOutput($name, $default='')
    {
        return (isset($this->_moduleOutputs[$name])) ? $this->_moduleOutputs[$name] : $default;
    }



}