<?php
/**
 * Contenido extension pack (ContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Frontend_PageCode
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2008-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


/**
 * Page code collection class. Places all module codes at the top of a article page and replaces 
 * the module container tags against the module outputs.
 *
 * The advantage of this page type code generation ist the possibility to process all page related
 * codes before starting any output to the client.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Frontend_PageCode
 * @author      Murat Purc <murat@purc.de>
 */
class ConMp_Pagecode_Container_Collect extends ConMp_Pagecode_Abstract
{

    /**
     * Self instance
     *
     * @var  ConMp_Pagecode_Container_Collect
     */
    private static $_instance;

    /**
     * To collect modulecodes
     * @var  string
     */
    private $_codeTop;


    /**
     * Prevent public instantiation
     */
    protected function __construct()
    {
        parent::__construct();
        $this->_codeTop = '';
    }


    /**
     * Disable cloning
     */
    private function __clone()
    {
    }


    /**
     * Returns self instance
     *
     * @return  ConMp_Pagecode_Container_Collect
     */
    public static function getInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }


    public function processCode()
    {
        if (!$this->_check()){
            throw new Exception('ConMp_Pagecode_Container_Collect->processCode() Wrong initialization');
        }

		foreach ($this->_layoutContainer as $key => $value) {

            // get layout container module
            $module = $this->_getLayoutContainerModule($value);

			// do replacements
            $this->_doContainerConfVarReplacements($module, $value);

			// do some debug related changes
            $this->_doModuleDebug($module, $value);

			// replace module container
            $this->_replaceContainers($module, $value);
        }

        return $this->_codeTop . $this->_code;
    }


    protected function _replaceContainers($module, $value)
    {
        // set module code
        $moduleOutput = '<?php
// ####################### start module "' . $value . '" #######################
' . $module->CiCMS_VALUE . '
ob_start();
?>
' . $module->output . '
<?php
$modOut = ob_get_contents();
ob_end_clean();
ConMp_FePage::getInstance()->addModuleOutput("' . $value . '", $modOut);
// ######################## end module "' . $value . '" ########################
?>';

        // add modulecode 
        $this->_codeTop .= $moduleOutput;

        $moduleReplacement = '<?php echo ConMp_FePage::getInstance()->getModuleOutput("' . $value . '"); ?>';

        $this->_code = preg_replace("/<container( +)id=\\\\\"$value\\\\\"(.*)>(.*)<\/container>/Uis", "CMS_CONTAINER[$value]", $this->_code);
        $this->_code = preg_replace("/<container( +)id=\\\\\"$value\\\\\"(.*)\/>/i", "CMS_CONTAINER[$value]", $this->_code);
        $this->_code = str_ireplace("CMS_CONTAINER[$value]", $moduleReplacement, $this->_code);
    }

}