<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Bootstrap
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Abstract plugin bootstrap class
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Bootstrap
 * @author      Murat Purc <murat@purc.de>
 */
abstract class ConMp_Plugin_Bootstrap_Abstract
{

    /**
     * Flag to store loaded status. Will be set to true in run() function.
     * @var  bool
     */
    protected $_run = false;

    /**
     * Configuration instance
     * @var  ConMp_Config
     */
     protected $_config;

    /**
     * Pluginname
     * @var  string
     */
    private $_pluginName;


    /**
     * Time taken
     * @var  string
     */
    private $_timeTaken = 0;


    /**
     * Prevent public instantiation.
     *
     * @param   string     $pluginName  The plugin name
     * @retun   void
     * @throws  Exception  If passed parameter plugin name is invalid
     */
    protected function __construct($pluginName)
    {
        if (trim($pluginName) == '') {
            throw new Exception('Invalid parameter $pluginName');
        }
        $this->_pluginName = $pluginName;

        plugin_include('mpContenidoX', 'libs/ConMp/Config.php');
        $this->_config = ConMp_Config::getInstance();
    }


    /**
     * Main function to call. Invokes all boostrap related functions.
     *
     * @retun  void
     */
    public function run()
    {
        if ($this->_run) {
            return;
        }
        $this->_run = true;

        $startTime = microtime(true);

        $this->_loadConfiguration();

        $methods = get_class_methods($this);
        foreach ($methods as $method) {
            if (5 < strlen($method) && '_init' === substr($method, 0, 5)) {
                #echo "$method<br />";
                $this->$method();
            }
        }

        // store time taken
        $this->_timeTaken = microtime(true) - $startTime;
    }

    
    /**
     * Returns time taken
     *
     * @return  float
     */
     public function getTimeTaken()
     {
        return $this->_timeTaken;
     }


    /**
     * Includes some needed sources.
     *
     * @retun  void
     */
    protected function _initIncludes()
    {
    }


    /**
     * Loads the main configuration from plugin include folder 
     * (contenido/plugins/{pluginname}/includes/config.plugin.{pluginname}.php).
     *
     * @retun   void
     * @throws  Exception  If configuration couldn't loaded
     */
    private function _loadConfiguration()
    {
        try {
            $path = $this->_config->path['contenido'] . $this->_config->path['plugins'] 
                  . $this->_pluginName . '/includes/';
            ConMp_Config::getInstance()->loadConfig($path, 'config.plugin.' . $this->_pluginName);
        } catch (Exception $e) {
            throw new Exception('Missing configuration file config.plugin.' . $this->_pluginName . '.php. ' . $e->getMessage());
        }
    }
    
}
