<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Request
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2008-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * HTTP Request class (very minimalistic version, stripped down to basics)
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Request
 * @author      Murat Purc <murat@purc.de>
 */
class ConMp_Request
{

    /**
     * Self instance
     * @var  ConMp_Request
     */
    private static $_instance = null;

    /**
     * Assoziative user parameter
     * @var  array
     */
    private $_userParams = array();


    /**
     * Constructor
     *
     * @return  void
     */
    protected function __construct()
    {
    }


    /**
     * Prevent cloning
     *
     * @return  void
     */
    private function __clone()
    {
        // donut
    }


    /**
     * Returns self instance.
     *
     * @return  ConMp_Request
     */
    public static function getInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }


    /**
     * Returns all parameter (user parameter + GET + POST + COOKIE)
     *
     * @return  assoziative array
     */
    public function getParams()
    {
        $params = $this->_userParams;
        if (isset($_GET) && is_array($_GET)) {
            $params = array_merge($params, $_GET);
        }
        if (isset($_POST) && is_array($_POST)) {
            $params = array_merge($params, $_POST);
        }
        if (isset($_COOKIE) && is_array($_COOKIE)) {
            $params = array_merge($params, $_COOKIE);
        }
        return $params;
    }


    /**
     * Returns the parameter
     *
     * @param   string  $key
     * @param   mixed   $default
     * @return  mixed
     */
    public function getParam($key, $default=null)
    {
        if (isset($this->_userParams[$key])) {
            return $this->_userParams[$key];
        } elseif (isset($_GET) && isset($_GET[$key])) {
            return $_GET[$key];
        } elseif (isset($_POST) && isset($_POST[$key])) {
            return $_POST[$key];
        } elseif (isset($_COOKIE) && isset($_COOKIE[$key])) {
            return $_COOKIE[$key];
        } else {
            return $default;
        }
    }


    /**
     * Sets the parameter
     *
     * @param   string  $key
     * @param   mixed   $value
     * @return  void
     */
    public function setParam($key, $value)
    {
        $this->_userParams[$key] = $value;
    }


    /**
     * Shortcut for ConMp_Request::getParam()
     *
     * @param   string  $key
     * @param   mixed   $default
     * @return  mixed
     */
    public function get($key, $default=null)
    {
        return $this->getParam($key, $default);
    }


    /**
     * Shortcut for ConMp_Request::setParam()
     *
     * @param   string  $key
     * @param   mixed   $value
     * @return  void
     */
    public function set($key, $value)
    {
        return $this->setParam($key, $value);
    }


    /**
     * Checks if the request method is "GET".
     *
     * @return  bool
     */
    public function isGet()
    {
        return (isset($_SERVER['REQUEST_METHOD']) && $_SERVER['REQUEST_METHOD'] == 'GET');
    }


    /**
     * Checks if the request method is "POST".
     *
     * @return  bool
     */
    public function isPost()
    {
        return (isset($_SERVER['REQUEST_METHOD']) && $_SERVER['REQUEST_METHOD'] == 'POST');
    }

}
