<?php
/**
 * Contenido extension pack (ContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     ContenidoX
 * @subpackage  i18n
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2008-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * i18n inplementation for Contenido, based on code of contenido/includes/functions.i18n.php
 *
 * Usage:
 * <code>
 * echo Contenido_i18n::getInstance()->i18n('message', 'contenido');
 * </code>
 *
 * @package     ContenidoX
 * @subpackage  i18n
 * @author      Murat Purc <murat@purc.de>
 */
class ConMp_i18n
{
    private static $_instance = null;
    private $_i18nTranslationCache = array();
    private $_transFiles = array();
    private $_gettextAvailable;
    private $_i18nLanguage;
    private $_emulateGettext = false;
    private $_cfg;

    protected function __construct()
    {
        $this->_cfg = & $GLOBALS['cfg'];

        // Auto initialization
        if (!isset($GLOBALS['belang'])) {
            if ($GLOBALS['contenido']) {
                // This is backend, we should trigger an error message here
                $stack = @debug_backtrace();
                $file = $stack[0]['file'];
                $line = $stack[0]['line'];
                cWarning ($file, $line, "i18nInit \$belang is not set");
            } // only send warning in backend	

            $GLOBALS['belang'] = false; // Needed - otherwise this won't work
        }  

        i18nInit($this->_cfg['path']['contenido'] . $this->_cfg['path']['locale'], $GLOBALS['belang']);
        $this->_i18nLanguage = $GLOBALS['i18nLanguage'];

        if (!isset($this->_cfg['native_i18n'])) {
            $this->_cfg['native_i18n'] = false;
        }

        $this->_gettextAvailable = (extension_loaded('gettext') && function_exists('dgettext'));

        $this->_emulateGettext = (!$this->_cfg['native_i18n'] || $this->_i18nLanguage == 'he_SS');
    }

    private function __clone()
    {
    }

    public function getInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    /**
     * gettext wrapper (for future extensions). Usage:
     * i18n("Your text which has to be translated");
     *
     * @param $string string The string to translate
     * @param $domain string The domain to look up
     * @return string  Returns the translation
     */
    public function i18n($string, $domain='contenido')
    {
        if ($this->_gettextAvailable && !$this->_emulateGettext) {
            if ($domain != 'contenido') {
                $translation = dgettext($domain, $string);
                return ($translation);
            } else {
                return gettext($string);
            }
        }

        return $this->i18nEmulateGettext($string, $domain);
    }


    /**
     * Emulates GNU gettext
     *
     * @param $string string The string to translate
     * @param $domain string The domain to look up
     * @return string  Returns the translation
     */
    function i18nEmulateGettext ($string, $domain='contenido')
    {
        global $i18nLanguage, $i18nDomains;

        if (isset($this->_i18nTranslationCache[$string])) {
            return $this->_i18nTranslationCache[$string];
        }

        // Bad thing, gettext is not available. Let's emulate it
        if (!file_exists($i18nDomains[$domain] . $i18nLanguage . '/LC_MESSAGES/' . $domain . '.po')) {
            return $string;
        }

        if (!isset($this->_transFiles[$domain])) {
            $this->_transFiles[$domain] = file_get_contents($i18nDomains[$domain] . $i18nLanguage . '/LC_MESSAGES/' . $domain . '.po');

            // Remove comments from file
            $this->_transFiles[$domain] = preg_replace('/^#.+/m', '', $this->_transFiles[$domain]);

            // Prepare for special po edit format 
            /* Something like: 
                #, php-format
                msgid ""
                "Hello %s,\n"
                "\n"
                "you've got a new reminder for the client '%s' at\n"
                "%s:\n"
                "\n"
                "%s"
                msgstr ""
                "Hallo %s,\n"
                "\n"
                "du hast eine Wiedervorlage erhalten fr den Mandanten '%s' at\n"
                "%s:\n"
                "\n"
                "%s"
                
               has to be converted to:
                msgid "Hello %s,\n\nyou've got a new reminder for the client '%s' at\n%s:\n\n%s"
                msgstr "Hallo %s,\n\ndu hast eine Wiedervorlage erhalten fr den Mandanten '%s' at\n%s:\n\n%s"
            */
            $this->_transFiles[$domain] = preg_replace('/\\\n"\\s+"/m', '\\\\n', $this->_transFiles[$domain]); 
            $this->_transFiles[$domain] = preg_replace('/(""\\s+")/m', '"', $this->_transFiles[$domain]);
        }

        $stringStart = strpos($this->_transFiles[$domain], '"' . str_replace(array("\n", "\r", "\t"), array('\n', '\r', '\t'), $string) . '"');
        if ($stringStart === false) {
            return $string;
        }

        $results = array();
        preg_match("/msgid.*\"(".preg_quote(str_replace(array("\n", "\r", "\t"), array('\n', '\r', '\t'), $string),"/").")\"(?:\s*)?\nmsgstr(?:\s*)\"(.*)\"/", $this->_transFiles[$domain], $results);
        # Old: preg_match("/msgid.*\"".preg_quote($string,"/")."\".*\nmsgstr(\s*)\"(.*)\"/", $this->_transFiles[$domain], $results);

        if (isset($results[1])) {
            $this->_i18nTranslationCache[$string] = stripslashes(str_replace(array('\n', '\r', '\t'), array("\n", "\r", "\t"), $results[2]));
            return $this->_i18nTranslationCache[$string];
        } else {
            return $string;	
        }
    }

}
